import { useGraphqlState } from "./useGraphqlState.js";
import { getEndpoint } from "./../helpers/composables.js";
import { hash } from "ohash";
import { GraphqlMiddlewareCache } from "../helpers/ClientCache.js";
import { useNuxtApp, useAppConfig } from "#imports";
function logGraphQLErrors(operation, operationName, errors) {
  errors.forEach((error) => {
    console.group(
      `Error in GraphQL response for ${operation} "${operationName}"`
    );
    console.error(`Message: ${error.message}`);
    if (error.locations && error.locations.length > 0) {
      const formattedLocations = error.locations.map((loc) => `line ${loc.line}, column ${loc.column}`).join(" | ");
      console.error(`Locations: ${formattedLocations}`);
    }
    if (error.path) {
      console.error(`Path: ${error.path.join(" -> ")}`);
    }
    console.groupEnd();
  });
}
export function performRequest(operation, operationName, method, options, cacheOptions) {
  const state = useGraphqlState();
  const app = useNuxtApp();
  if (!state) {
    console.error(
      `A GraphQL composable for ${operation} "${operationName}" was called before the "nuxt-graphql-middleware-provide-state" plugin could provide the state, which might lead to unexpected behaviour. Make sure that custom plugins that perform GraphQL requests are executed after "nuxt-graphql-middleware-provide-state" by setting it as a dependency via "dependsOn".`
    );
  }
  const key = `${operation}:${operationName}:${hash(options.params)}`;
  if (cacheOptions) {
    const config = useAppConfig();
    if (import.meta.client && cacheOptions.client && config.graphqlMiddleware.clientCacheEnabled) {
      if (!app.$graphqlCache) {
        app.$graphqlCache = new GraphqlMiddlewareCache(
          config.graphqlMiddleware.clientCacheMaxSize
        );
      }
      const cached = app.$graphqlCache.get(key);
      if (cached) {
        return cached;
      }
    }
  }
  const promise = $fetch(
    getEndpoint(operation, operationName),
    {
      ...state && state.fetchOptions ? state.fetchOptions : {},
      ...options,
      method
    }
  ).then((v) => {
    if (import.meta.dev && v.errors?.length) {
      logGraphQLErrors(operation, operationName, v.errors);
    }
    return {
      ...v,
      data: v.data,
      errors: v.errors || []
    };
  });
  if (import.meta.client && cacheOptions?.client && app.$graphqlCache) {
    app.$graphqlCache.set(key, promise);
  }
  return promise;
}
